from variables import *
from opcua import Server, ua
from time import sleep
from sys import exit
from os.path import isfile

def lecture_fichier(file):
    retour_dict = {}
    for line in file.readlines():
        var_name, var_value = line.split("=")
        retour_dict[var_name] = eval(var_value)
    return retour_dict

def lecture_premiere_ligne(file):
    retour_dict = {}
    file.seek(0)
    line = file.readline()
    try:
        var_name, var_value = line.split("=")
        retour_dict[var_name] = eval(var_value)
    except:
        print("Erreur de lecture du fichier")
        print(f"La ligne suivante n'a pas pu être lue : {line}")
        print("Les valeurs de retour ne sont pas modifiées")
        return None
    return retour_dict

def ecriture_fichier(liste_commandes):
    with open(SAVE_FILE_COMMAND, "w") as com:
        com.write(f"{liste_commandes}")

def actualisation_valeurs(
    fichier_pompes,
    fichier_reservoirs,
    variables_retour_pompes,
    variables_retour_reservoirs,
    variables_commandes_pompes,
):
    d = None
    # Mise a disposition des valeurs de retour des pompes
    while d is None:
        d = lecture_premiere_ligne(fichier_pompes)
    liste_retours_pompes = d["liste_retours_pompes"]
    for k, val in enumerate(liste_retours_pompes):
        # La modification est faite directement sur la variable
        variables_retour_pompes[k].set_value(val)

    d = None
    # Mise a disposition des valeurs de retour des reservoirs
    while d is None:
        d = lecture_premiere_ligne(fichier_reservoirs)
    liste_retours_reservoirs = d["liste_retours_reservoirs"]
    for k, val in enumerate(liste_retours_reservoirs[2:]):
        # La modification est faite directement sur la variable
        variables_retour_reservoirs[k].set_value(val)

    # Transmission des nouvelles commandes a la simulation
    liste_commandes_pompes = [0.0] * len(variables_commandes_pompes)
    for k, var_com in enumerate(variables_commandes_pompes):
        liste_commandes_pompes[k] = var_com.get_value()

    ecriture_fichier(liste_commandes_pompes)

def main():
    # Vérification de l'existence du fichier de sauvegarde
    if not isfile(SAVE_FILE_POMPES) or not isfile(SAVE_FILE_RESERVOIRS):
        print(
            f'Un des fichiers de sauvegarde n\'existe pas. Veuillez lancer la simulation "simulation_usine_eau.py" afin de le créer.'
        )
        print("Le programme va se fermer")
        exit()

    with open(SAVE_FILE_POMPES, "r") as retours_pompes:
        dict_retours_pompes = lecture_fichier(retours_pompes)

    with open(SAVE_FILE_RESERVOIRS, "r") as retours_reservoirs:
        dict_retours_reservoirs = lecture_fichier(retours_reservoirs)

    # Instanciation du serveur
    server = Server()
    # URL de la Raspberrypi qui héberge le serveur
    server.set_endpoint("opc.tcp://192.168.1.12:4840/UA/ServerOPCUA_ChateauDeau")
    # Nom et compagnie
    server.set_server_name("OPC-UA-Server")
    # Sécurité
    server.set_security_policy([ua.SecurityPolicyType.Basic256Sha256_SignAndEncrypt])
        #[ua.SecurityPolicyType.Basic256Sha256_SignAndEncrypt])
		#[ua.SecurityPolicyType.NoSecurity])
    server.load_certificate("certificate2.pem")
    server.load_private_key("key2.pem")
    # Nom de l'espace d'adresse pour éviter les ambiguïtés de noms de noeuds
    name = "Usine de production d'eau potable"
    noeud_usine = server.register_namespace(name)
    # Création d'un noeud racine
    node = server.get_objects_node()

    # Définition des variables pour les pompes
    n_pompes = len(dict_retours_pompes["liste_noms_pompes"])
    n_vars_par_pompe = len(dict_retours_pompes["liste_vars_pompes"]) // n_pompes
    variables_retours_pompes = []
    variables_commandes_pompes = []

    for k, name in enumerate(dict_retours_pompes["liste_noms_pompes"]):
        # Ajout d'un noeud pour les variables que l'ont veut partager
        pompe = node.add_object(noeud_usine, name)
        # Définition des variables
        for j in range(n_vars_par_pompe):
            handle = pompe.add_variable(
                noeud_usine,
                dict_retours_pompes["liste_vars_pompes"][k * n_vars_par_pompe + j],
                0,
            )
            variables_retours_pompes.append(handle)
        handle = pompe.add_variable(noeud_usine, "Commande", 0)
        handle.set_writable()
        variables_commandes_pompes.append(handle)

    # Définition des variables pour les réservoirs
    n_reservoirs = len(dict_retours_reservoirs["liste_noms_reservoirs"])
    n_vars_par_reservoir = (
        len(dict_retours_reservoirs["liste_vars_reservoirs"]) - 2
    ) // n_reservoirs
    variables_retours_reservoirs = []

    for k, name in enumerate(dict_retours_reservoirs["liste_noms_reservoirs"]):
        # Ajout d'un noeud pour les variables que l'ont veut partager
        reservoir = node.add_object(noeud_usine, name)
        # Définition des variables
        for j in range(n_vars_par_reservoir):
            handle = reservoir.add_variable(
                noeud_usine,
                dict_retours_reservoirs["liste_vars_reservoirs"][
                    2 + k * n_vars_par_reservoir + j
                ],
                0,
            )
            variables_retours_reservoirs.append(handle)

    # Demarrage du serveur
    try:
        server.start()
        with open(SAVE_FILE_POMPES, "r") as retours_pompes, open(
            SAVE_FILE_RESERVOIRS, "r"
        ) as retours_reservoirs:
            while True:
                actualisation_valeurs(
                    retours_pompes,
                    retours_reservoirs,
                    variables_retours_pompes,
                    variables_retours_reservoirs,
                    variables_commandes_pompes,
                )
                sleep(1)
    except KeyboardInterrupt:
        print(f"Fermeture du serveur OPC")
        server.stop()

if __name__ == "__main__":
    main()
