import random as rd
from variables import *
from time import sleep, time
import logging

logging.basicConfig(filename="simu.log", encoding="utf-8", level=logging.INFO)


class Pompe:
    def __init__(self, name, Q_MAX):
        self.name = name
        self.fonctionne = True
        self.debit = None
        self.commande_debit = None
        self.correction = 1
        self.Q_MAX = Q_MAX

    def get_debit(self):
        return f"{self.debit}"

    def get_etat(self):
        return f"{self.fonctionne}"

    def get_data(self):
        return [self.get_debit(), self.get_etat()]

    def get_nom(self):
        return [self.name]

    def get_nom_var(self):
        return ["Debit", "Etat"]

    def set_debit(self, q):
        if q < 0:
            self.debit = 0
        elif q > self.Q_MAX:
            self.debit = self.Q_MAX
        else:
            self.debit = q

    def marche(self):
        self.set_debit(self.commande_debit * self.correction * self.Q_MAX / 100)

    def commande(self, q):
        # Commande en %
        self.commande_debit = q
        self.marche()

    def arret(self):
        self.debit = 0

    def defaut_pompe(self):
        choix = rd.randint(1, 3)
        self.fonctionne = False
        if choix == 1:
            # panne pompe
            self.arret()
        elif choix == 2:
            # pompe trop lente
            self.correction = 0.5
            self.marche()
        else:
            # pompe trop rapide
            self.correction = 1.5
            self.marche()

    def repare(self):
        self.fonctionne = True
        self.correction = 1
        self.marche()


class Reservoir:
    def __init__(self, name, V_MAX):
        self.name = name
        self.volume = 0
        self.concentration_chlore = C_REF_CHLORE
        self.debit_entrant = 0
        self.debit_sortant = 0
        self.V_MAX = V_MAX
        self.debit_chlore = 0
        self.concentration_entrante = 0

    def __repr__(self) -> str:
        text = f"Reservoir {self.name} : \n"
        text += f"Volume : {self.volume} m^3 \n"
        text += f"Concentration chlore : {self.concentration_chlore} mg/L \n"
        text += f"Debit entrant : {self.debit_entrant} m^3/h \n"
        text += f"Debit sortant : {self.debit_sortant} m^3/h \n"
        text += f"Debit chlore : {self.debit_chlore} m^3/h \n\n"
        return text

    def get_volume(self):
        return f"{self.volume}"

    def get_concentration(self):
        return f"{self.concentration_chlore}"

    def get_debit_entrant(self):
        return f"{self.debit_entrant}"

    def get_debit_sortant(self):
        return f"{self.debit_sortant}"

    def get_debit_chlore(self):
        return f"{self.debit_chlore}"

    def get_data(self):
        return [
            self.get_volume(),
            self.get_concentration(),
            self.get_debit_entrant(),
            self.get_debit_sortant(),
            self.get_debit_chlore(),
            f"{self.V_MAX}",
        ]

    def get_nom(self):
        return [self.name]

    def get_nom_var(self):
        return [
            "Volume",
            "Concentration",
            "Debit Entrant",
            "Debit Sortant",
            "Debit Chlore",
            "Volume Max",
        ]

    def set_debit_entrant(self, debit):
        self.debit_entrant = debit

    def set_debit_sortant(self, debit):
        self.debit_sortant = debit

    def set_debit_chlore(self, debit):
        self.debit_chlore = debit

    def verification_volume(self, volume):
        # Return True si le volume du réservoir est au dessus de la limite
        return self.volume >= volume

    def trop_plein(self):
        return self.volume < self.V_MAX

    def actualisation(self, dt):
        v_entrant = self.debit_entrant * dt / 3600
        v_sortant = self.debit_sortant * dt / 3600
        v_entrant_chlore = self.debit_chlore * dt / 3600

        n_apport_chlore = (
            v_entrant_chlore * C_SOLUTION_CHLORE
            + v_entrant * self.concentration_entrante
        )
        n_depart_chlore = v_sortant * self.concentration_chlore
        n_restant_chlore = (
            self.volume * self.concentration_chlore - n_depart_chlore + n_apport_chlore
        )

        if n_restant_chlore < 0:
            n_restant_chlore = 0
        new_volume = self.volume + v_entrant + v_entrant_chlore - v_sortant
        if new_volume < 0:
            new_volume = 0
        self.volume = new_volume
        if self.volume != 0:
            self.concentration_chlore = n_restant_chlore / self.volume


class DataCommande:
    def __init__(self, l_pompes):
        self.liste_pompes = l_pompes
        self.liste_commandes = [0.0] * len(l_pompes)
        with open(SAVE_FILE_COMMAND, "w") as file:
            file.write(f"{self.liste_commandes}")

    def actualiser(self):
        with open(SAVE_FILE_COMMAND, "r") as file:
            txt = file.read()
            # Tres mauvaise idée, il vaut mieux utiliser un parser
            # style YAML ou JSON, mais je n'ai pas le temps de changer ça
            try:
                self.liste_commandes = eval(txt)
            except:
                print("Erreur de lecture du fichier de commande")
                print(f"Contenu du fichier : {txt}")
                print("La commande n'a pas été modifiée")


class DataRetour:
    def __init__(self, l_pompes, l_reservoirs):
        self.liste_pompes = l_pompes
        self.liste_reservoirs = l_reservoirs
        self.liste_retours_pompes = None
        self.liste_retours_reservoirs = None
        self.liste_noms_pompes = []
        self.liste_noms_reservoirs = []
        self.liste_vars_reservoirs = ["t", "dt"]
        self.liste_vars_pompes = []
        self.liste_vmax_reservoirs = []
        for pompe in self.liste_pompes:
            self.liste_noms_pompes += pompe.get_nom()
            self.liste_vars_pompes += pompe.get_nom_var()
        for reservoir in self.liste_reservoirs:
            self.liste_noms_reservoirs += reservoir.get_nom()
            self.liste_vars_reservoirs += reservoir.get_nom_var()

    def fill(self, t, dt):
        self.liste_retours_pompes = []
        self.liste_retours_reservoirs = [f"{t}", f"{dt}"]
        for pompe in self.liste_pompes:
            self.liste_retours_pompes += pompe.get_data()
        for reservoir in self.liste_reservoirs:
            self.liste_retours_reservoirs += reservoir.get_data()

    def repr_pompes(self):
        return f"""liste_retours_pompes=[{', '.join(self.liste_retours_pompes)}]
liste_vars_pompes=["{'", "'.join(self.liste_vars_pompes)}"]
liste_noms_pompes=["{'", "'.join(self.liste_noms_pompes)}"]"""

    def repr_reservoirs(self):
        return f"""liste_retours_reservoirs=[{', '.join(self.liste_retours_reservoirs)}]
liste_vars_reservoirs=["{'", "'.join(self.liste_vars_reservoirs)}"]
liste_noms_reservoirs=["{'", "'.join(self.liste_noms_reservoirs)}"]
"""


def main():
    pompe_captage = Pompe("Pompe Captage", Q_MAX_CAPTAGE)
    # pompe_filtrage = Pompe("Pompe Filtrage", Q_MAX_FILTRAGE)
    pompe_chlore = Pompe("Pompe Chlore", Q_MAX_CHLORE)
    pompe_relevage = Pompe("Pompe Relevage", Q_MAX_RELEVAGE)
    reservoir_desinfection = Reservoir(
        "Reservoir Desinfection", V_MAX_RESERVOIR_DESINFECTION
    )
    reservoir_desinfection.concentration_chlore = 0.0
    reservoir_desinfection.volume = V_MAX_RESERVOIR_DESINFECTION
    chateau_d_eau = Reservoir("Chateau d eau", V_MAX_CHATEAU)
    chateau_d_eau.volume = V_INITIAL_CHATEAU
    retours = DataRetour(
        # [pompe_captage, pompe_filtrage, pompe_chlore, pompe_relevage],
        [pompe_captage, pompe_chlore, pompe_relevage],
        [reservoir_desinfection, chateau_d_eau],
    )
    commandes = DataCommande(
        # [pompe_captage, pompe_filtrage, pompe_chlore, pompe_relevage],
        [pompe_captage, pompe_chlore, pompe_relevage],
    )

    start = time()
    t_old = start
    try:
        while True:
            # Commande
            tik = time()
            commandes.actualiser()
            pompe_captage.commande(commandes.liste_commandes[0])
            # pompe_filtrage.commande(commandes.liste_commandes[1])
            pompe_chlore.commande(commandes.liste_commandes[1])
            pompe_relevage.commande(commandes.liste_commandes[2])
            reservoir_desinfection.set_debit_chlore(pompe_chlore.debit)
            reservoir_desinfection.set_debit_entrant(pompe_captage.debit)
            reservoir_desinfection.set_debit_sortant(pompe_relevage.debit)
            chateau_d_eau.set_debit_entrant(pompe_relevage.debit)
            chateau_d_eau.set_debit_sortant(50)
            chateau_d_eau.concentration_entrante = (
                reservoir_desinfection.concentration_chlore
            )
            dt = tik - t_old
            reservoir_desinfection.actualisation(dt)
            logging.debug(f"{pompe_relevage.debit}")
            chateau_d_eau.actualisation(dt)
            # Retour
            retours.fill(tik - start, dt)
            save = open(SAVE_FILE_POMPES, "w")
            save.write(f"{retours.repr_pompes()}")
            save.close()
            save = open(SAVE_FILE_RESERVOIRS, "w")
            save.write(f"{retours.repr_reservoirs()}")
            save.close()
            t_old = tik
            tok = time()
            pause = PERIODE_RAFRAICHISSEMENT - (tok - tik)
            if pause < 0:
                logging.warning(
                    "Période de rafraîchissement trop faible, le processeur ne suit plus"
                )
                pause = 0
            sleep(pause)
    except KeyboardInterrupt:
        print("Le programme s'est arrêté correctement")


if __name__ == "__main__":
    main()
